@@ -5,12 +5,11 @@ Used from markdown.py
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-
 import base64
 import mimetypes
 import os
 from html import escape
-from typing import Any, Callable, Dict, Iterable, Match, Optional, Tuple
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Iterable, Match, Optional, Protocol, Tuple
 
 import bs4
 from pygments import highlight
@@ -21,8 +20,21 @@ from pygments.util import ClassNotFound
 
 from nbconvert.filters.strings import add_anchor
 
+if TYPE_CHECKING:
+    try:
+        from mistune.plugins import Plugin
+    except ImportError:
+
+        class Plugin(Protocol):  # type: ignore[no-redef]
+            """Mistune plugin interface."""
+
+            def __call__(self, markdown: "Markdown") -> None:
+                """Apply the plugin on the markdown document."""
+                ...
+
+
 try:  # for Mistune >= 3.0
-    from mistune import (
+    from mistune import (  # type:ignore[attr-defined]
         BlockParser,
         BlockState,
         HTMLRenderer,
@@ -33,6 +45,7 @@ try:  # for Mistune >= 3.0
     )
 
     MISTUNE_V3 = True
+    MISTUNE_V3_ATX = "atx_heading" in BlockParser.SPECIFICATION
 
 except ImportError:  # for Mistune >= 2.0
     import re
@@ -46,17 +59,16 @@ except ImportError:  # for Mistune >= 2.0
     )
 
     MISTUNE_V3 = False
+    MISTUNE_V3_ATX = False
 
-    def import_plugin(name: str) -> 'MarkdownPlugin':  # type: ignore[misc]
+    def import_plugin(name: str) -> "Plugin":  # type: ignore[misc]
         """Simple implementation of Mistune V3's import_plugin for V2."""
         return PLUGINS[name]  # type: ignore[no-any-return]
 
 
-class InvalidNotebook(Exception):  # noqa
+class InvalidNotebook(Exception):
     """An invalid notebook model."""
 
-    pass
-
 
 def _dotall(pattern: str) -> str:
     """Makes the '.' special character match any character inside the pattern, including a newline.
@@ -78,8 +90,10 @@ if MISTUNE_V3:  # Parsers for Mistune >= 3.0.0
         is ignored here.
         """
 
-        AXT_HEADING_WITHOUT_LEADING_SPACES = (
-            r"^ {0,3}(?P<axt_1>#{1,6})(?!#+)(?P<axt_2>[ \t]*(.*?)?)$"
+        ATX_HEADING_WITHOUT_LEADING_SPACES = (
+            r"^ {0,3}(?P<atx_1>#{1,6})(?!#+)(?P<atx_2>[ \t]*(.*?)?)$"
+            if MISTUNE_V3_ATX
+            else r"^ {0,3}(?P<axt_1>#{1,6})(?!#+)(?P<axt_2>[ \t]*(.*?)?)$"
         )
 
         MULTILINE_MATH = _dotall(
@@ -95,12 +109,14 @@ if MISTUNE_V3:  # Parsers for Mistune >= 3.0.0
 
         SPECIFICATION = {
             **BlockParser.SPECIFICATION,
-            "axt_heading": AXT_HEADING_WITHOUT_LEADING_SPACES,
+            (
+                "atx_heading" if MISTUNE_V3_ATX else "axt_heading"
+            ): ATX_HEADING_WITHOUT_LEADING_SPACES,
             "multiline_math": MULTILINE_MATH,
         }
 
         # Multiline math must be searched before other rules
-        DEFAULT_RULES: Tuple[str, ...] = ("multiline_math", *BlockParser.DEFAULT_RULES)  # type: ignore[assignment]
+        DEFAULT_RULES: ClassVar[Iterable[str]] = ("multiline_math", *BlockParser.DEFAULT_RULES)  # type: ignore[assignment]
 
         def parse_multiline_math(self, m: Match[str], state: BlockState) -> int:
             """Send mutiline math as a single paragraph to MathInlineParser."""
@@ -142,7 +158,7 @@ if MISTUNE_V3:  # Parsers for Mistune >= 3.0.0
         }
 
         # Block math must be matched first, and all math must come before text
-        DEFAULT_RULES: Tuple[str, ...] = (
+        DEFAULT_RULES: ClassVar[Iterable[str]] = (
             "block_math_tex",
             "block_math_latex",
             "inline_math_tex",
@@ -202,7 +218,7 @@ else:  # Parsers for Mistune >= 2.0.0 < 3.0.0
         AXT_HEADING = re.compile(r" {0,3}(#{1,6})(?!#+)(?: *\n+|([^\n]*?)(?:\n+|\s+?#+\s*\n+))")
 
         # Multiline math must be searched before other rules
-        RULE_NAMES = ("multiline_math", *BlockParser.RULE_NAMES)  # type: ignore
+        RULE_NAMES = ("multiline_math", *BlockParser.RULE_NAMES)  # type: ignore[attr-defined]
 
         def parse_multiline_math(self, m: Match[str], state: Any) -> Dict[str, str]:
             """Pass token through mutiline math."""
@@ -234,7 +250,7 @@ else:  # Parsers for Mistune >= 2.0.0 < 3.0.0
             "inline_math_tex",
             "inline_math_latex",
             "latex_environment",
-            *InlineParser.RULE_NAMES,  # type: ignore
+            *InlineParser.RULE_NAMES,  # type: ignore[attr-defined]
         )
 
         def parse_block_math_tex(self, m: Match[str], state: Any) -> Tuple[str, str]:
@@ -268,7 +284,7 @@ else:  # Parsers for Mistune >= 2.0.0 < 3.0.0
 class IPythonRenderer(HTMLRenderer):
     """An ipython html renderer."""
 
-    def __init__(  # noqa
+    def __init__(
         self,
         escape: bool = True,
         allow_harmful_protocols: bool = True,
@@ -277,6 +293,7 @@ class IPythonRenderer(HTMLRenderer):
         anchor_link_text: str = "¶",
         path: str = "",
         attachments: Optional[Dict[str, Dict[str, str]]] = None,
+        **lexer_options,
     ):
         """Initialize the renderer."""
         super().__init__(escape, allow_harmful_protocols)
@@ -284,6 +301,7 @@ class IPythonRenderer(HTMLRenderer):
         self.exclude_anchor_links = exclude_anchor_links
         self.anchor_link_text = anchor_link_text
         self.path = path
+        self.lexer_options = lexer_options
         if attachments is not None:
             self.attachments = attachments
         else:
@@ -301,7 +319,7 @@ class IPythonRenderer(HTMLRenderer):
             try:
                 if info.strip().split(None, 1):
                     lang = info.strip().split(maxsplit=1)[0]
-                    lexer = get_lexer_by_name(lang, stripall=True)
+                    lexer = get_lexer_by_name(lang, **self.lexer_options)
             except ClassNotFound:
                 code = f"{lang}\n{code}"
                 lang = None
@@ -402,7 +420,7 @@ class IPythonRenderer(HTMLRenderer):
             default_mime_type = next(iter(attachment.keys()))
             return f"data:{default_mime_type};base64,{attachment[default_mime_type]}"
 
-        elif self.embed_images:
+        if self.embed_images:
             base64_url = self._src_to_base64(src)
             if base64_url is not None:
                 return base64_url
@@ -445,10 +463,6 @@ class IPythonRenderer(HTMLRenderer):
         return str(parsed_html)
 
 
-# Represents an already imported plugin for Mistune
-MarkdownPlugin = Callable[[Markdown], None]
-
-
 class MarkdownWithMath(Markdown):
     """Markdown text with math enabled."""
 
@@ -467,7 +481,7 @@ class MarkdownWithMath(Markdown):
         renderer: HTMLRenderer,
         block: Optional[BlockParser] = None,
         inline: Optional[InlineParser] = None,
-        plugins: Optional[Iterable[MarkdownPlugin]] = None,
+        plugins: Optional[Iterable["Plugin"]] = None,
     ):
         """Initialize the parser."""
         if block is None:
@@ -476,7 +490,7 @@ class MarkdownWithMath(Markdown):
             if MISTUNE_V3:
                 inline = MathInlineParser(hard_wrap=False)
             else:
-                inline = MathInlineParser(renderer, hard_wrap=False)  # type: ignore
+                inline = MathInlineParser(renderer, hard_wrap=False)  # type: ignore[arg-type,misc]
         if plugins is None:
             plugins = (import_plugin(p) for p in self.DEFAULT_PLUGINS)
 
